@@ -4,13 +4,14 @@ import subprocess
 import sys
 from dataclasses import dataclass, FrozenInstanceError
 from math import sqrt
+from enum import Enum
 from types import SimpleNamespace
-from typing import List, NamedTuple, Tuple, Optional, Dict, Set
+from typing import List, NamedTuple, Tuple, Optional, Dict, Set, Type, Any
 from contextlib import redirect_stderr
 
 import pytest
 
-from smart_arg import arg_suite, custom_arg_suite, LateInit, SmartArgError, TypeHandler, PrimitiveHandlerAddon
+from smart_arg import arg_suite, custom_arg_suite, LateInit, SmartArgError, TypeHandler, _first_handles
 from smart_arg_mock import main, MockArgTup
 
 
@@ -191,23 +192,29 @@ def test_argv():
 
 
 def test_primitive_addon():
-    class IntHandlerAddon(PrimitiveHandlerAddon):
+    class IntHandlerAddon:
         @staticmethod
-        def build_primitive(arg_type, kwargs):
-            if arg_type is int:
-                kwargs.type = lambda s: int(s) ** 2
+        def build_type(arg_type) -> Any:
+            return lambda s: int(s) ** 2
 
         @staticmethod
-        def str(arg) -> str:
+        def build_str(arg) -> str:
             return str(int(sqrt(arg)))
 
-        handled_types = [int]
+        @staticmethod
+        def handles(t: Type) -> bool:
+            return t == int
 
     class IntTypeHandler(TypeHandler):
-        def _build_common(self, kwargs, field_meta):
+        def _build_common(self, kwargs, field_meta) -> None:
+            super()._build_common(kwargs, field_meta)
             kwargs.help = '(int, squared)'
 
-        handled_types = [int]
+        def _build_other(self, kwargs, arg_type) -> None:
+            kwargs.type = _first_handles(self.primitive_addons, arg_type).build_type(arg_type)
+
+        def handles(self, t: Type) -> bool:
+            return t == int
 
     @custom_arg_suite(primitive_handler_addons=[IntHandlerAddon], type_handlers=[IntTypeHandler])
     class MyTuple(NamedTuple):
@@ -348,3 +355,35 @@ def test_dataclass():
         frozen: bool = True
 
     pytest.raises(FrozenInstanceError, NoPostInit().mutate)  # mutation not allowed after init
+
+
+def test_basic_enum():
+    class Color(Enum):
+        RED = 1
+        BLUE = 2
+        GREEN = 3
+
+    @arg_suite
+    class MyEnumBasic(NamedTuple):
+        a_int: int
+        my_color_dict: Dict[int, Color]
+        my_color_list: List[Color]
+        my_color_tuple: Tuple[Color, int]
+        default_color: Color = Color.RED
+
+    arg_cmd = ['--a_int', '1', '--my_color_dict', '10:RED', '20:BLUE', '--my_color_list', 'GREEN',
+               '--my_color_tuple', 'BLUE', '100', '--default_color', 'GREEN']
+    basic_tup = MyEnumBasic(a_int=1, my_color_dict={10: Color.RED, 20: Color.BLUE}, my_color_list=[Color.GREEN],
+                            my_color_tuple=(Color.BLUE, 100), default_color=Color.GREEN)
+
+    parsed_tup:MyEnumBasic = MyEnumBasic.__from_argv__(arg_cmd)
+    assert basic_tup == parsed_tup
+    seriliazed_cmd_line = basic_tup.__to_argv__()
+    assert set(seriliazed_cmd_line) == set(arg_cmd)
+    my_parser = MyEnumBasic.__arg_suite__._parser
+    assert my_parser._option_string_actions['--my_color_dict'].metavar == "int:<enum 'Color'>"
+    assert my_parser._option_string_actions['--default_color'].choices == list(Color)
+
+    arg_cmd2 = ['--a_int', '1', '--my_color_dict', '10:red', '--my_color_list', 'GREEN', '--my_color_tuple', 'BLUE',
+                '100']
+    pytest.raises(SmartArgError, MyEnumBasic.__from_argv__, arg_cmd2)
